require 'sketchup.rb'

# Ruby essentials
# http://www.techotopia.com/index.php/Ruby_Essentials
# http://www.zenspider.com/Languages/Ruby/QuickRef.html
# http://www.regular-expressions.info/ruby.html

# SketchUp
# http://sketchup.google.com
# http://www.suwiki.org
# http://www.sketchucation.com/forums/scf/viewforum.php?f=15

# SketchUp Ruby API
# http://download.sketchup.com/OnlineDoc/gsu6_ruby/Docs/index.html
# http://code.google.com/apis/sketchup/docs/developers_guide/index.html
# http://www.crai.archi.fr/rubylibrarydepot/ruby/offset.rb
# http://www.smustard.com/script/Offset
# http://www.sketchucation.com/forums/scf/

# G-Code
# http://en.wikipedia.org/wiki/G-code

# Groups & Forums
# http://www.phlatforum.com/
# http://thecrashcast.com/
# http://crash-hancock.blogspot.com/
# http://www.vimeo.com/user945475/videos   Michael Hancock's videos
# http://www.rcgroups.com/forums/showthread.php?t=888387   Introducing The Phlatprinter * Available for Purchase *
# http://www.rcflightcast.com/
# http://allthingsthatfly.com/

# Translation
# Portugus (ECM) - marcomartim@yahoo.com.br

# Name Begins With Variable Scope 
# $  A global variable  
# @  An instance variable  
# [a-z] or _  A local variable  
# [A-Z]  A constant 
# @@ A class variable 

# - - - - - - - - - - - - - - - - -
#           Default Values
# - - - - - - - - - - - - - - - - -
$default_file_name = "gcode.cnc"
$default_file_ext = "cnc"
$default_directory_name = Dir.pwd + "/"

$default_spindle_speed = 15000
$default_feed_rate = 2000.0.mm
$default_plunge_rate = 1500.0.mm
$default_safe_travel = 2.2.mm
$default_material_thickness = 6.3.mm
$default_cut_depth_factor = 100
$default_bit_diameter = 3.mm
$default_tab_width = 2.8.mm
$default_tab_depth_factor = 50
$default_vtabs = false
$default_fold_depth_factor = 50

$default_safe_origin_x = 0.mm
$default_safe_origin_y = 0.mm
$default_safe_width = 1000.mm
$default_safe_height = 550.mm
$default_comment_remark = ""

$default_overhead_gantry = false
$default_multipass = false
$default_multipass_depth = 1.mm

# - - - - - - - - - - - - - - - - -
#           Cursor image files
# - - - - - - - - - - - - - - - - -
$cursor_directory = "Tools/Phlatboyz/images"
$cursor_tab_tool = "cursor_tabtool.png"
$cursor_vtab_tool = "cursor_vtabtool.png"
$cursor_inside_cuttool_filename = "cursor_cuttool_inside.png"
$cursor_outside_cuttool_filename = "cursor_cuttool_outside.png"
$cursor_fold_tool = "cursor_foldtool.png"
$cursor_safe_tool = "cursor_safetool.png"
$cursor_centerline_tool = "cursor_centerlinetool.png"
$cursor_plunge_tool = "cursor_plungetool.png"

# - - - - - - - - - - - - - - - - -
#           Dictionary Keys
# - - - - - - - - - - - - - - - - -
$dict_name = "phlatboyzdictionary"
$dict_spindle_speed = "spindle_speed"
$dict_feed_rate = "feed_rate"
$dict_plunge_rate = "plunge_rate"
$dict_safe_travel = "safe_travel"
$dict_material_thickness = "material_thickness"
$dict_cut_depth_factor = "cut_depth_factor"
$dict_output_file_name = "output_file_name"
$dict_output_directory_name = "output_directory_name"
$dict_cut_depth_factor = "cut_depth_factor"
$dict_edge_type = "edge_type"
$dict_object_mark = "object_mark"
$dict_tab_width = "tab_width"
$dict_bit_diameter = "bit_diameter"
$dict_fold_depth_factor = "fold_depth_factor"
$dict_cut_depth_factor = "cut_depth_factor"
$dict_edge_count = "edge_count"
$dict_tab_depth_factor = "tab_depth_factor"
$dict_vtabs = "vtabs"
$dict_tab_edge_type = "tab_edge_type"
$dict_comment_text = "Comment_Remark"
$dict_safe_origin_x = "safe_origin_x"
$dict_safe_origin_y = "safe_origin_y"
$dict_safe_width = "safe_width"
$dict_safe_height = "safe_height"
$dict_phlatarc_center = "arc_center"
$dict_phlatarc_radius = "arc_radius"
$dict_phlatarc_angle = "arc_angle"
$dict_phlatarc_g3 = "arc_g3"
$dict_vtab = "vtab"
$dict_multipass = "multipass"
$dict_multipass_depth = "multipass_depth"
$dict_overhead_gantry = "overhead_gantry"

$dict_construction_mark = "construction_mark"

# - - - - - - - - - - - - - - - - -
#           Cut Keys
# - - - - - - - - - - - - - - - - -

$key_inside_cut = "inside_cut"
$key_outside_cut = "outside_cut"
$key_fold_cut = "fold_cut"
$key_centerline_cut = "centerline_cut"
$key_tab_cut = "tab_cut"
$key_plunge_cut = "plunge_cut"

$key_reg_cut_arr = [$key_inside_cut, $key_outside_cut, $key_tab_cut]

# - - - - - - - - - - - - - - - - -
#         misc  parameters
# - - - - - - - - - - - - - - - - -
$construction_font_height = 10.mm
$min_z = -1.4
$max_z = 1.4

$fold_shorten_width = 0.0625.inch
$fold_depth_factor_array = [25, 50, 75, 100]

$reverse_loop_direction = false
$reflection_output = false

# http://download.sketchup.com/OnlineDoc/gsu6_ruby/Docs/ruby-color.html
$color_inside_cut = "DeepSkyBlue"
$color_outside_cut = "Orange"
$color_cut_drawing = "red"
$color_fold_cut = "Fuchsia"
$color_fold_wide_cut = "MediumVioletRed"

$color_tab_drawing = "Green"
$color_tab_cut = "Green"
$color_vtab_drawing = "DarkOrchid"
$color_vtab_cut = "DarkOrchid"

$color_safe_drawing = "blue"
$color_centerline_cut = "DarkSeaGreen"
$color_plunge_cut = "Brown"

ver_ar = Sketchup.version.split(".")
if (ver_ar) && (ver_ar.length > 0) && (ver_ar[0].to_i == 7) && (ver_ar[1].to_i < 1)
  $rendering_edge_color_mode = 3
elsif (ver_ar) && (ver_ar.length > 0) && (ver_ar[0].to_i < 7)
  $rendering_edge_color_mode = 3  
else
  $rendering_edge_color_mode = 0
end

# -------------------------
# PhlatScript Features
# -------------------------

# Set this to true if you have problems with the parameter dialog being blank or crashing SU
$use_compatible_dialogs = false

# Set this to true to enable multipass fields in the parameters dialog. When it is false
# you will not be prompted to use multipass. When true you will be able to turn it off and
# on in the parameters dialog
$use_multipass = true

# Set this to true if you have an older version of Mach that does not slow down
# to the Z maximum speed during helical linear interpolation (G2/3 with Z 
# movement A.K.A vtabs on an arc). vtabs on arcs will cut at the plunge rate 
# defined in this file or overriden in the parameters dialog
$use_vtab_speed_limit = false

# Set this to true to use G61. This will make the machine come to a complete
# stop when changing directions instead of rounding out square corners. When 
# set to false the default for your CNC software will be used. Without G61
# the machine will maintain the best possible speed for the cut even if the 
# tool isn't true to the cut path. Rounded corners at low feedrates aren't 
# very noticeable but anything over 200 starts to generate large radii so 
# that the momentum of the machine can be maintained.
$use_exact_path = true

module PhlatScript
  PB_MENU_TOOLBAR = 1
  PB_MENU_MENU = 2
  PB_MENU_CONTEXT = 4
end